#mine {
	color: red;
	border-color: rgb(10, 100, 230);
	border-color: rgba(10, 100, 231, 0.3);
	outline-color: #222;
	background-color: #232323;
}

#yours {
	background-color: hsl(220, 10%, 220%);
	background-color: hsla(220, 10%, 220%, 0.3);
}
