body {
  font-size: 2em; font-family: Helvetica,Arial,sans-serif; font-weight: bold;
  border-width: 2px; border-color: #999; border-style: dotted;
  background-color: #fff; background-image: url('foobar.png'); background-repeat: repeat-y;
  margin-top: 2px; margin-right: 3px; margin-bottom: 4px; margin-left: 5px;
}
