#header {
	margin: 10px 2em 1cm 2%;
	font-family: Verdana, Helvetica, "Gill Sans", sans-serif;
	font-size: 10px;
	color: red !important;
	background-color: green;
	background-color: rgba(0,128,0,0.7);
	frequency: 30Hz;
}

body {
	color: green;
	font: 75% "Lucida Grande", "Trebuchet MS", Verdana, sans-serif;
}
